package org.redhat.gss.decision.server.client;

import java.io.IOException;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;

import org.drools.compiler.runtime.pipeline.impl.DroolsJaxbHelperProviderImpl;
import org.drools.core.command.impl.GenericCommand;
import org.drools.core.command.runtime.BatchExecutionCommandImpl;
import org.drools.core.command.runtime.rule.FireAllRulesCommand;
import org.drools.core.command.runtime.rule.InsertObjectCommand;
import org.kie.api.command.BatchExecutionCommand;
import org.kie.internal.KnowledgeBase;
import org.kie.internal.builder.KnowledgeBuilder;
import org.kie.internal.builder.KnowledgeBuilderFactory;
import org.kie.internal.builder.help.KnowledgeBuilderHelper;
import org.kie.internal.io.ResourceFactory;

import com.esrx.ref.testproject.Bean1;
import com.sun.tools.xjc.Language;
import com.sun.tools.xjc.Options;

public class DecisionClient {

	public static void main(String args[]) throws ClassNotFoundException, JAXBException, IOException {

	
		//common part for all approaches
		Bean1 bean1 = new Bean1();
		bean1.setName("Robert");

		InsertObjectCommand insertObjectCommand = new InsertObjectCommand(bean1,"f1");
		FireAllRulesCommand fireAllRulesCommand = new FireAllRulesCommand("myFireCommand");
		List<GenericCommand<?>> commands = new ArrayList<GenericCommand<?>>();
		commands.add(insertObjectCommand);
		commands.add(fireAllRulesCommand);

		BatchExecutionCommand command = new BatchExecutionCommandImpl(commands);

		// approach 1 - works
		List<String> classNames = new ArrayList<String>();
		classNames.add("com.esrx.ref.testproject.Bean1");
		JAXBContext jaxbContext = DroolsJaxbHelperProviderImpl.createDroolsJaxbContext(classNames, null);
		Marshaller marshaller = jaxbContext.createMarshaller();

		StringWriter xml = new StringWriter();
		marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
		marshaller.marshal(command, xml);

		System.out.println(xml);
		System.out.println("===================================");
		
		// approach 2 - customer
	
		Options xjcOpts = new Options();
		xjcOpts.setSchemaLanguage(Language.XMLSCHEMA);			
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();			
		String[] classNames2 = KnowledgeBuilderHelper.addXsdModel( ResourceFactory.newClassPathResource( "testjaxb.xsd",
				com.esrx.ref.testproject.Bean1.class ),
		                                                           kbuilder,
		                                                           xjcOpts,
		                                                           "xsd" );
		
		for (String str: classNames2) {
			
			System.out.println(str);
		}
		
		KnowledgeBase kbase = kbuilder.newKnowledgeBase();
		JAXBContext jaxbContext2 = KnowledgeBuilderHelper.newJAXBContext(classNames2, kbase);		
	
		
		Marshaller marshaller2 = jaxbContext2.createMarshaller();
		StringWriter xml2 = new StringWriter();
		marshaller2.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
		marshaller2.marshal(command, xml2);	
		System.out.println("========================");
		System.out.println(xml2);
	}

}
